# =======================
# = Documentation tasks =
# =======================
begin
  require 'yard'
  require 'yard/rake/yardoc_task'
  
  task :documentation => :'documentation:generate'
  namespace :documentation do
    YARD::Rake::YardocTask.new :generate do |t|
      t.files   = ['lib/**/*.rb', 'ext/**/*.c']
      t.options = ['--output-dir', File.join('meta', 'documentation'),
                   '--readme', 'README']
    end
    
    YARD::Rake::YardocTask.new :dotyardoc do |t|
      t.files   = ['lib/**/*.rb', 'ext/**/*.c']
      t.options = ['--no-output',
                   '--readme', 'README']
    end
    
    class Numeric
      def pretty_inspect(decimal_points = 3)
        bits = self.to_s.split('.')
        bits[0] = bits[0].reverse.scan(/\d{1,3}/).join(',').reverse
        bits[1] = bits[1][0...decimal_points] if bits[1]
        bits.join('.')
      end
    end
    
    task :verify do
      documentation_threshold = 50.0
      doc = YARD::CLI::Yardoc.new; doc.generate = false; doc.run
      
      percent_documented = (
        YARD::Registry.all.select {|o| !o.docstring.empty? }.size /
        YARD::Registry.all.size.to_f
      ) * 100
      puts "Documentation coverage: #{percent_documented.pretty_inspect(1)}% (threshold: #{documentation_threshold.pretty_inspect(1)}%)"
    end
    
    task :open do
      system 'open ' + File.join('meta', 'documentation', 'index.html') if PLATFORM['darwin']
    end
  end
  
rescue LoadError
  desc 'You need the `yard` gem to generate documentation'
  task :documentation
end